/*
Copyright (C) 2011 Craciun Dan <craciun.dan@tuxarena.com>

This file is part of NeonView.

NeonView is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

NeonView is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with NeonView.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "header.h"

void put_window (TM *tm) {
	tm->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	if (strcmp(tm->tcfg->maximized, "yes") == 0) {
		gtk_window_maximize(GTK_WINDOW(tm->window));
	} else {
		gtk_window_set_default_size(GTK_WINDOW(tm->window), tm->tcfg->window_width, tm->tcfg->window_height);
	}
	gtk_widget_set_size_request(tm->window, 512, 200);
	gtk_window_set_position(GTK_WINDOW(tm->window), tm->tcfg->gtk_win_pos);
	gtk_window_set_title(GTK_WINDOW(tm->window), tm->tcfg->window_title);
	gtk_container_set_border_width(GTK_CONTAINER(tm->window), 0);

	GdkPixbuf *icon = gdk_pixbuf_new_from_file("icon.png", NULL);

	gtk_window_set_icon(GTK_WINDOW(tm->window), icon);

	g_signal_connect_swapped(tm->window, "delete-event", G_CALLBACK(gtk_my_quit), tm);
}

void put_vbox (TM *tm) {
	tm->vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(tm->window), tm->vbox);
}

void put_menubar (TM *tm) {
	GtkWidget *menufile, *file, *open, *sep1, *quit;
	GtkWidget *menuimage, *image, *sep3, *size_fit, *size_normal;
	GtkWidget *menutools, *tools, *settings;
	GtkWidget *menuhelp, *help, *manual, *homepage, *sep2, *about;
	GtkWidget *sep4, *zoom_plus, *zoom_minus;

	GtkWidget *sep5, *rot_left, *rot_right, *rot_flip;

	//	GtkWidget *sep6, *info;

	GtkAccelGroup *accel_group;

	tm->menubar = gtk_menu_bar_new();

	accel_group = gtk_accel_group_new();
	gtk_window_add_accel_group(GTK_WINDOW(tm->window), accel_group);

	/*--- FILE MENU ---*/

	menufile = gtk_menu_new();

	file = gtk_menu_item_new_with_mnemonic("_File");
	open = gtk_menu_item_new_with_mnemonic("_Open");
	//	sep4 = gtk_separator_menu_item_new();
	//	save_as = gtk_menu_item_new_with_mnemonic("Save _As...");
	sep1 = gtk_separator_menu_item_new();
	quit = gtk_menu_item_new_with_mnemonic("_Quit");

	gtk_menu_item_set_submenu(GTK_MENU_ITEM(file), menufile);
	gtk_menu_shell_append(GTK_MENU_SHELL(menufile), open);
	//	gtk_menu_shell_append(GTK_MENU_SHELL(menufile), sep4);
	//	gtk_menu_shell_append(GTK_MENU_SHELL(menufile), save_as);
	gtk_menu_shell_append(GTK_MENU_SHELL(menufile), sep1);
	gtk_menu_shell_append(GTK_MENU_SHELL(menufile), quit);
	gtk_menu_shell_append(GTK_MENU_SHELL(tm->menubar), file);

	gtk_widget_add_accelerator(open, "activate", accel_group, GDK_KEY_o, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	//	gtk_widget_add_accelerator(save_as, "activate", accel_group, GDK_KEY_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(quit, "activate", accel_group, GDK_KEY_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(quit, "activate", accel_group, GDK_KEY_Escape, 0, GTK_ACCEL_VISIBLE);

	g_signal_connect_swapped(open, "activate", G_CALLBACK(put_open), tm);
	//	g_signal_connect_swapped(save_as, "activate", G_CALLBACK(put_save_as), tm);
	g_signal_connect_swapped(quit, "activate", G_CALLBACK(gtk_my_quit), tm);

	/*--- IMAGE MENU ---*/

	menuimage = gtk_menu_new();

	image = gtk_menu_item_new_with_mnemonic("_Image");
	tm->menu_next = gtk_menu_item_new_with_mnemonic("_Next");
	tm->menu_prev = gtk_menu_item_new_with_mnemonic("_Previous");
	sep3 = gtk_separator_menu_item_new();
	size_normal = gtk_menu_item_new_with_mnemonic("Normal _Size");
	size_fit = gtk_menu_item_new_with_mnemonic("_Fit to Window");
	sep4 = gtk_separator_menu_item_new();
	zoom_plus = gtk_menu_item_new_with_mnemonic("Zoom _In");
	zoom_minus = gtk_menu_item_new_with_mnemonic("Zoom _Out");
	sep5 = gtk_separator_menu_item_new();
	rot_left = gtk_menu_item_new_with_mnemonic("Rotate _Left");
	rot_right = gtk_menu_item_new_with_mnemonic("Rotate _Right");
	rot_flip = gtk_menu_item_new_with_mnemonic("_Flip Image");

	//	sep6 = gtk_separator_menu_item_new();
	//	info = gtk_menu_item_new_with_mnemonic("Image I_nformation");

	gtk_menu_item_set_submenu(GTK_MENU_ITEM(image), menuimage);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), tm->menu_next);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), tm->menu_prev);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), sep3);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), size_normal);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), size_fit);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), sep4);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), zoom_plus);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), zoom_minus);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), sep5);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), rot_left);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), rot_right);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), rot_flip);
	//	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), sep6);
	//	gtk_menu_shell_append(GTK_MENU_SHELL(menuimage), info);
	gtk_menu_shell_append(GTK_MENU_SHELL(tm->menubar), image);

	gtk_widget_add_accelerator(tm->menu_next, "activate", accel_group, GDK_KEY_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(tm->menu_next, "activate", accel_group, GDK_KEY_space, 0, GTK_ACCEL_MASK);
	gtk_widget_add_accelerator(tm->menu_next, "activate", accel_group, GDK_KEY_Left, 0, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(tm->menu_prev, "activate", accel_group, GDK_KEY_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(tm->menu_prev, "activate", accel_group, GDK_KEY_BackSpace, 0, GTK_ACCEL_MASK);
	gtk_widget_add_accelerator(tm->menu_prev, "activate", accel_group, GDK_KEY_Right, 0, GTK_ACCEL_MASK);
	gtk_widget_add_accelerator(size_normal, "activate", accel_group, GDK_KEY_0, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(size_fit, "activate", accel_group, GDK_KEY_9, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

	gtk_widget_add_accelerator(zoom_plus, "activate", accel_group, GDK_KEY_equal, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(zoom_minus, "activate", accel_group, GDK_KEY_minus, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(rot_left, "activate", accel_group, GDK_KEY_Left, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(rot_right, "activate", accel_group, GDK_KEY_Right, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator(rot_flip, "activate", accel_group, GDK_KEY_Up, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

	//	gtk_widget_add_accelerator(info, "activate", accel_group, GDK_KEY_i, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

	g_signal_connect_swapped(tm->menu_next, "activate", G_CALLBACK(show_next), tm);
	g_signal_connect_swapped(tm->menu_prev, "activate", G_CALLBACK(show_prev), tm);
	g_signal_connect_swapped(size_normal, "activate", G_CALLBACK(zoom_normal), tm);
	g_signal_connect_swapped(size_fit, "activate", G_CALLBACK(zoom_fit), tm);

	g_signal_connect_swapped(zoom_plus, "activate", G_CALLBACK(zoom_in), tm);
	g_signal_connect_swapped(zoom_minus, "activate", G_CALLBACK(zoom_out), tm);
	g_signal_connect_swapped(rot_left, "activate", G_CALLBACK(rotate_right), tm);
	g_signal_connect_swapped(rot_right, "activate", G_CALLBACK(rotate_left), tm);
	g_signal_connect_swapped(rot_flip, "activate", G_CALLBACK(rotate_flip), tm);

	//	g_signal_connect_swapped(info, "activate", G_CALLBACK(put_info), tm);

	/*--- TOOLS MENU ---*/

	menutools = gtk_menu_new();

	tools = gtk_menu_item_new_with_mnemonic("_Tools");
	settings = gtk_menu_item_new_with_mnemonic("_Settings");

	gtk_menu_item_set_submenu(GTK_MENU_ITEM(tools), menutools);
	gtk_menu_shell_append(GTK_MENU_SHELL(menutools), settings);
	gtk_menu_shell_append(GTK_MENU_SHELL(tm->menubar), tools);

	g_signal_connect_swapped(settings, "activate", G_CALLBACK(put_settings), tm);

	gtk_widget_add_accelerator(settings, "activate", accel_group, GDK_KEY_t, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

	/*--- HELP MENU ---*/

	menuhelp = gtk_menu_new();

	help = gtk_menu_item_new_with_mnemonic("_Help");
	manual = gtk_menu_item_new_with_mnemonic("_Manual");
	homepage = gtk_menu_item_new_with_mnemonic("_Homepage");
	sep2 = gtk_separator_menu_item_new();
	about = gtk_menu_item_new_with_mnemonic("_About");

	gtk_menu_item_set_submenu(GTK_MENU_ITEM(help), menuhelp);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuhelp), manual);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuhelp), homepage);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuhelp), sep2);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuhelp), about);
	gtk_menu_shell_append(GTK_MENU_SHELL(tm->menubar), help);

	gtk_widget_add_accelerator(manual, "activate", accel_group, GDK_KEY_F1, 0, GTK_ACCEL_VISIBLE);

	g_signal_connect_swapped(manual, "activate", G_CALLBACK(put_manual), tm);
	g_signal_connect_swapped(homepage, "activate", G_CALLBACK(put_homepage), tm);
	g_signal_connect_swapped(about, "activate", G_CALLBACK(put_about), tm);

	gtk_box_pack_start(GTK_BOX(tm->vbox), tm->menubar, FALSE, FALSE, 0);
}

void put_toolbar (TM *tm) {
	GtkToolItem *open;
	GtkToolItem *sep1;
	//	GtkToolItem *prev;
	//	GtkToolItem *next;
	GtkToolItem *sep2;
	GtkToolItem *sep3;
	GtkToolItem *manual;
	GtkToolItem *about;
	GtkToolItem *settings;
	GtkToolItem *sep4;
	GtkToolItem *quit;
	GtkToolItem *sep5;
	GtkToolItem *sep6, *zoom_minus;
	GtkToolItem *zoom_plus;

	GtkToolItem *rot_right, *rot_left, *rot_flip;
	//	GtkToolItem *sep7;

	tm->toolbar = gtk_toolbar_new();

	gtk_toolbar_set_style(GTK_TOOLBAR(tm->toolbar), GTK_TOOLBAR_ICONS);

	open = gtk_tool_button_new_from_stock(GTK_STOCK_OPEN);
	gtk_tool_item_set_tooltip_text(open, "Open an image file");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), open, -1);
	g_signal_connect_swapped(open, "clicked", G_CALLBACK(put_open), tm);

	sep1 = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), sep1, -1);

	tm->prev = gtk_tool_button_new_from_stock(GTK_STOCK_GO_BACK);
	gtk_tool_item_set_tooltip_text(tm->prev, "Display previous image in directory");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), tm->prev, -1);
	g_signal_connect_swapped(tm->prev, "clicked", G_CALLBACK(show_prev), tm);

	tm->next = gtk_tool_button_new_from_stock(GTK_STOCK_GO_FORWARD);
	gtk_tool_item_set_tooltip_text(tm->next, "Display next image in directory");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), tm->next, -1);
	g_signal_connect_swapped(tm->next, "clicked", G_CALLBACK(show_next), tm);

	sep2 = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), sep2, -1);

	tm->normal_size = gtk_tool_button_new_from_stock(GTK_STOCK_ZOOM_100);
	gtk_tool_item_set_tooltip_text(tm->normal_size, "View image at normal size");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), tm->normal_size, -1);
	g_signal_connect_swapped(tm->normal_size, "clicked", G_CALLBACK(zoom_normal), tm);

	tm->fit_size = gtk_tool_button_new_from_stock(GTK_STOCK_ZOOM_FIT);
	gtk_tool_item_set_tooltip_text(tm->fit_size, "Fit image to window");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), tm->fit_size, -1);
	g_signal_connect_swapped(tm->fit_size, "clicked", G_CALLBACK(zoom_fit), tm);

	sep6 = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), sep6, -1);

	zoom_minus = gtk_tool_button_new_from_stock(GTK_STOCK_ZOOM_OUT);
	gtk_tool_item_set_tooltip_text(zoom_minus, "Zoom Out");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), zoom_minus, -1);
	g_signal_connect_swapped(zoom_minus, "clicked", G_CALLBACK(zoom_out), tm);

	zoom_plus = gtk_tool_button_new_from_stock(GTK_STOCK_ZOOM_IN);
	gtk_tool_item_set_tooltip_text(zoom_plus, "Zoom In");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), zoom_plus, -1);
	g_signal_connect_swapped(zoom_plus, "clicked", G_CALLBACK(zoom_in), tm);

	//	sep7 = gtk_separator_tool_item_new();
	//	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), sep7, -1);

	rot_left = gtk_tool_button_new_from_stock(GTK_STOCK_GOTO_FIRST);
	gtk_tool_item_set_tooltip_text(rot_left, "Rotate Left");
	//	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), rot_left, -1);
	g_signal_connect_swapped(rot_left, "clicked", G_CALLBACK(rotate_right), tm);

	rot_right = gtk_tool_button_new_from_stock(GTK_STOCK_GOTO_LAST);
	gtk_tool_item_set_tooltip_text(rot_right, "Rotate Right");
	//	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), rot_right, -1);
	g_signal_connect_swapped(rot_right, "clicked", G_CALLBACK(rotate_left), tm);

	rot_flip = gtk_tool_button_new_from_stock(GTK_STOCK_GOTO_BOTTOM);
	gtk_tool_item_set_tooltip_text(rot_flip, "Flip");
	//	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), rot_flip, -1);
	g_signal_connect_swapped(rot_flip, "clicked", G_CALLBACK(rotate_flip), tm);

	sep3 = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), sep3, -1);

	settings = gtk_tool_button_new_from_stock(GTK_STOCK_PREFERENCES);
	gtk_tool_item_set_tooltip_text(settings, "Configure NeonView");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), settings, -1);
	g_signal_connect_swapped(settings, "clicked", G_CALLBACK(put_settings), tm);

	sep4 = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), sep4, -1);

	manual = gtk_tool_button_new_from_stock(GTK_STOCK_HELP);
	gtk_tool_item_set_tooltip_text(manual, "Open the NeonView manual");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), manual, -1);
	g_signal_connect_swapped(manual, "clicked", G_CALLBACK(put_manual), tm);

	about = gtk_tool_button_new_from_stock(GTK_STOCK_ABOUT);
	gtk_tool_item_set_tooltip_text(about, "Show information about NeonView");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), about, -1);
	g_signal_connect_swapped(about, "clicked", G_CALLBACK(put_about), tm);

	sep5 = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), sep5, -1);

	quit = gtk_tool_button_new_from_stock(GTK_STOCK_QUIT);
	gtk_tool_item_set_tooltip_text(quit, "Quit NeonView");
	gtk_toolbar_insert(GTK_TOOLBAR(tm->toolbar), quit, -1);
	g_signal_connect_swapped(quit, "clicked", G_CALLBACK(gtk_my_quit), tm);

	gtk_box_pack_start(GTK_BOX(tm->vbox), tm->toolbar, FALSE, FALSE, 0);
}

void put_hbox1 (TM *tm) {
	tm->hbox1 = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(tm->vbox), tm->hbox1, TRUE, TRUE, 0);
}

/* TODO: left sidebar code goes here */

void put_scrolled (TM *tm) {
	tm->scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(tm->scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	gtk_box_pack_start(GTK_BOX(tm->hbox1), tm->scrolled, TRUE, TRUE, 0);
}

void put_hbox2 (TM *tm) {
	tm->hbox2 = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(tm->vbox), tm->hbox2, FALSE, FALSE, 2);
}

void put_label1 (TM *tm) {
	tm->label1 = gtk_label_new("No image loaded");
	gtk_label_set_max_width_chars(GTK_LABEL(tm->label1), 32);
	gtk_widget_set_size_request(tm->label1, 360, 18);
	gtk_box_pack_start(GTK_BOX(tm->hbox2), tm->label1, FALSE, FALSE, 10);

	tm->label2 = gtk_label_new("");
	gtk_label_set_max_width_chars(GTK_LABEL(tm->label2), 11);
	gtk_widget_set_size_request(tm->label2, 80, 0);
	gtk_box_pack_start(GTK_BOX(tm->hbox2), tm->label2, FALSE, FALSE, 10);

	tm->label3 = gtk_label_new("");
	gtk_label_set_max_width_chars(GTK_LABEL(tm->label3), 12);
	gtk_widget_set_size_request(tm->label3, 80, 0);
	gtk_box_pack_start(GTK_BOX(tm->hbox2), tm->label3, FALSE, FALSE, 10);

	tm->label4 = gtk_label_new("");
	gtk_label_set_max_width_chars(GTK_LABEL(tm->label4), 10);
	gtk_widget_set_size_request(tm->label4, 80, 0);
	gtk_box_pack_start(GTK_BOX(tm->hbox2), tm->label4, FALSE, FALSE, 10);

	tm->label5 = gtk_label_new("");
	gtk_label_set_max_width_chars(GTK_LABEL(tm->label5), 20);
	gtk_widget_set_size_request(tm->label5, 100, 0);
	gtk_box_pack_start(GTK_BOX(tm->hbox2), tm->label5, FALSE, FALSE, 10);
}

void put_open (TM *tm) {
	GtkWidget *dialog;
	gchar *filename;
	int emit;

	dialog = gtk_file_chooser_dialog_new("Open File", NULL, GTK_FILE_CHOOSER_ACTION_OPEN,
										 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
										 GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);

	emit = gtk_dialog_run(GTK_DIALOG(dialog));

	if (emit == GTK_RESPONSE_OK) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog)); // full pathname to the file
		gtk_widget_destroy(dialog);

		/*--- get the dirname from the full path (filename) ---*/

		char dirname[4097];
		int save = -1;
		int i = strlen(filename);

		while (save == -1) {
			if (filename[i] == '/') {
				save = i;
			}
			i--;
		}

		for (i = 0; i < save; i++) {
			dirname[i] = filename[i];
		}
		dirname[i] = '\0';

		strcpy(tm->tfd->path, dirname);
		strcpy(tm->tfd->disp, filename);
		put_image(tm);

		chdir(dirname);
		getcwd(tm->tfd->cwd, sizeof(tm->tfd->cwd));
		strcpy(tm->tfd->listd, tm->tfd->cwd);
		list_files(tm);
		sort_files(tm);

		/* now set the position of the opened file to increase/decrease on next/prev */
		if (tm->tfd->total == 0 || tm->tfd->total == 1) {
			gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->next), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), FALSE);
		}
		if (tm->tfd->total > 1) {
			if (tm->tfd->pos == 0) {
			gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->next), TRUE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), TRUE);
			}
			if (tm->tfd->pos == tm->tfd->total - 1) {
			gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), TRUE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), TRUE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->next), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), FALSE);
			}
		}

		for (i = 0; i <  tm->tfd->total; i++) {
			if (strcmp(tm->tfd->disp, tm->tfd->files[i]) == 0) {
				tm->tfd->pos = i;
			}
		}

	} else {
		gtk_widget_destroy(dialog);
	}

	if (tm->tfd->pos == 0) {
		put_image(tm);
		gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
	}
}

void show_next (TM *tm) {
	(tm->tfd->pos)++;
	char tmp[4097]; int j = 0;

	if (tm->tfd->pos < tm->tfd->total) {
		strcpy(tm->tfd->disp, tm->tfd->files[tm->tfd->pos]);
		/*
		for (int i = strlen(tm->tfd->disp) - 1; i > strlen(tm->tfd->cwd); i--) {
			tmp[j] = tm->tfd->disp[i];
			j++;
		}
		tmp[j] = '\0';
		rts(tmp);
		*/
		int i = strlen(tm->tfd->files[tm->tfd->pos]) - 1;
		int save = -1;
		j = 0;

		while (save == -1) {
			if (tm->tfd->files[tm->tfd->pos][i] == '/') {
				save = i;
			} else {
				tmp[j] = tm->tfd->files[tm->tfd->pos][i];
				j++;
			}
			i--;
		}
		tmp[j] = '\0';
		rts(tmp);

		strcpy(tm->tfd->basename, tmp);

		put_image(tm);
	}
	if (tm->tfd->pos == tm->tfd->total - 1) {
		/*		char tmp[4097]; int j = 0;
		for (int i = strlen(tm->tfd->disp) - 1; i > strlen(tm->tfd->cwd); i--) {
			tmp[j] = tm->tfd->disp[i];
			j++;
		}
		tmp[j] = '\0';
		rts(tmp);
		*/
		int i = strlen(tm->tfd->files[tm->tfd->pos]) - 1;
		int save = -1;
		j = 0;

		while (save == -1) {
			if (tm->tfd->files[tm->tfd->pos][i] == '/') {
				save = i;
			} else {
				tmp[j] = tm->tfd->files[tm->tfd->pos][i];
				j++;
			}
			i--;
		}
		tmp[j] = '\0';
		rts(tmp);

		strcpy(tm->tfd->basename, tmp);

		put_image(tm);
		gtk_widget_set_sensitive(GTK_WIDGET(tm->next), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), FALSE);
	}
	if (tm->tfd->pos > 0) {
		gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), TRUE);
	}
	if (tm->fit_images == 1) {
		strcpy(tm->tfd->disp, tm->tfd->files[tm->tfd->pos]);
		zoom_fit(tm);
		return;
	}
}

void show_prev (TM *tm) {
	(tm->tfd->pos)--;
	char tmp[4097]; int j = 0;

	if (tm->tfd->pos > 0) {
		strcpy(tm->tfd->disp, tm->tfd->files[tm->tfd->pos]);

		int i = strlen(tm->tfd->files[tm->tfd->pos]) - 1;
		int save = -1;
		j = 0;

		while (save == -1) {
			if (tm->tfd->files[tm->tfd->pos][i] == '/') {
				save = i;
			} else {
				tmp[j] = tm->tfd->files[tm->tfd->pos][i];
				j++;
			}
			i--;
		}
		tmp[j] = '\0';
		rts(tmp);

		strcpy(tm->tfd->basename, tmp);

		put_image(tm);
	}
	if (tm->tfd->pos == 0) {
		strcpy(tm->tfd->disp, tm->tfd->files[tm->tfd->pos]);

		int i = strlen(tm->tfd->files[tm->tfd->pos]) - 1;
		int save = -1;
		j = 0;

		while (save == -1) {
			if (tm->tfd->files[tm->tfd->pos][i] == '/') {
				save = i;
			} else {
				tmp[j] = tm->tfd->files[tm->tfd->pos][i];
				j++;
			}
			i--;
		}
		tmp[j] = '\0';
		rts(tmp);

		strcpy(tm->tfd->basename, tmp);

		put_image(tm);
		gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
	}
	if (tm->tfd->pos < tm->tfd->total - 1) {
		gtk_widget_set_sensitive(GTK_WIDGET(tm->next), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), TRUE);
	}
	if (tm->fit_images == 1) {
		strcpy(tm->tfd->disp, tm->tfd->files[tm->tfd->pos]);
		zoom_fit(tm);
		return;
	}
}

void zoom_normal (TM *tm) {
	tm->fit_images = 0;
	gtk_label_set_text(GTK_LABEL(tm->label5), "Normal Size");

	if (tm->image != NULL) {
		gtk_widget_destroy(tm->image);
	}
	tm->pixbuf = gdk_pixbuf_new_from_file(tm->tfd->disp, NULL);
	tm->image = gtk_image_new_from_pixbuf(tm->pixbuf);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(tm->scrolled), tm->image);

	tm->tfi->new_width = gdk_pixbuf_get_width(tm->pixbuf);
	tm->tfi->new_height = gdk_pixbuf_get_height(tm->pixbuf);

	gtk_widget_show(tm->image);
	gtk_widget_show_all(tm->scrolled);
}

void zoom_fit (TM *tm) {
	int width, height;
	float ratio;

	if (strcmp(tm->tcfg->fit_next, "yes") == 0 || strcmp(tm->tcfg->remember_fit, "yes") == 0) {
		tm->fit_images = 1;
	} else {
		tm->fit_images = 0;
	}

	gtk_label_set_text(GTK_LABEL(tm->label5), "Fit to Window");

	gtk_window_get_size(GTK_WINDOW(tm->window), &width, &height);

	if (tm->image != NULL) {
		gtk_widget_destroy(tm->image);
	}
	tm->pixbuf = gdk_pixbuf_new_from_file(tm->tfd->disp, NULL);

	if (width - 20 > tm->tfi->width) {
		if (height - 92 > tm->tfi->height) {
			tm->image = gtk_image_new_from_pixbuf(tm->pixbuf);
			gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(tm->scrolled), tm->image);
		}
		if (height - 92 <= tm->tfi->height) {
			ratio = tm->tfi->height / (float)(height - 92);
			tm->pixbuf = gdk_pixbuf_scale_simple(GDK_PIXBUF(tm->pixbuf), tm->tfi->width / ratio, tm->tfi->height / ratio, GDK_INTERP_BILINEAR);
			tm->image = gtk_image_new_from_pixbuf(tm->pixbuf);
			gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(tm->scrolled), tm->image);
		}
	}
	if (width - 20 <= tm->tfi->width) {
		if (height - 92 > tm->tfi->height) {
			ratio = tm->tfi->width / (float)(width - 20);
			tm->pixbuf = gdk_pixbuf_scale_simple(GDK_PIXBUF(tm->pixbuf), tm->tfi->width / ratio, tm->tfi->height / ratio, GDK_INTERP_BILINEAR);
			tm->image = gtk_image_new_from_pixbuf(tm->pixbuf);
			gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(tm->scrolled), tm->image);
		}
		if (height - 92 <= tm->tfi->height) {
			if (tm->tfi->width > tm->tfi->height) {
				ratio = tm->tfi->width / (float)(width - 20);
			} else {
				ratio = tm->tfi->height / (float)(height - 92);
			}

			/* trick is the new size for height or width still couldn't fit */
			float new_width = tm->tfi->width / ratio;
			float new_height = tm->tfi->height / ratio;

			if (new_height >= height - 92) {
				ratio = new_height / (float)(height - 92);
			} else if (new_width >= width - 20) {
				ratio = new_width / (float)(width - 20);
			}

			tm->pixbuf = gdk_pixbuf_scale_simple(GDK_PIXBUF(tm->pixbuf), new_width / ratio, new_height / ratio, GDK_INTERP_BILINEAR);
			tm->image = gtk_image_new_from_pixbuf(tm->pixbuf);

	tm->tfi->new_width = gdk_pixbuf_get_width(tm->pixbuf);
	tm->tfi->new_height = gdk_pixbuf_get_height(tm->pixbuf);

			gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(tm->scrolled), tm->image);
		}
	}

	gtk_widget_show(tm->image);
	gtk_widget_show_all(tm->scrolled);
}

void demo_cb (TM *tm) {
	fprintf(stdout, "Callback demo_cb\n");
}

void show_widgets (TM *tm) {
	put_window(tm);
	put_vbox(tm);
	put_menubar(tm);
	put_toolbar(tm);
	put_hbox1(tm);
	put_scrolled(tm);
	put_hbox2(tm);
	put_label1(tm);

	gtk_widget_show_all(tm->vbox);
	gtk_widget_show(tm->window);
}
